<?php

namespace Drp\NativeEventApp\FileUploads;

use Carbon\Carbon;
use Drp\NativeEventApp\FileUploads\Contracts\FileUploadDriver;
use Drp\NativeEventApp\Models\FileUpload;
use Illuminate\Contracts\Filesystem\FileNotFoundException;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\URL;

class LocalDriver implements FileUploadDriver
{
    public function createUrl(string $uuid, string $expiry): array
    {
        $url = URL::temporarySignedRoute(
            'eventpal.sdk.file-upload',
            Carbon::parse($expiry),
            ['uuid' => $uuid]
        );

        return [
            'url' => $url,
            'headers' => json_encode([]),
        ];
    }

    public function storeReference(string $uuid, int $registrationId, ?string $type = null): FileUpload
    {
        return FileUpload::create([
            'uuid' => $uuid,
            'type' => $type ?? null,
            'disk' => config('native-event-app.files.local.disk'),
            'path' => $this->getFilePath($uuid),
            'registration_id' => $registrationId,
        ]);
    }

    private function getFilePath(string $uuid): string
    {
        $diskPath = config('native-event-app.files.local.path') . '/' . $uuid;

        throw_unless(
            Storage::disk(config('native-event-app.files.local.disk'))->exists($diskPath),
            new FileNotFoundException("File does not exist at path {$diskPath}")
        );

        return $diskPath;
    }
}
