<?php

namespace Drp\NativeEventApp\FileUploads;

use Aws\CommandInterface;
use Aws\S3\S3Client;
use Drp\NativeEventApp\FileUploads\Contracts\FileUploadDriver;
use Drp\NativeEventApp\Models\FileUpload;
use Illuminate\Contracts\Filesystem\FileNotFoundException;

class S3Driver implements FileUploadDriver
{
    public function createUrl(string $uuid, string $expiry): array
    {
        $client = $this->storageClient();

        $signedRequest = $client->createPresignedRequest(
            $this->createCommand($client, $uuid),
            $expiry
        );

        return [
            'url' => (string) $signedRequest->getUri(),
            'headers' => json_encode(
                array_merge($signedRequest->getHeaders(), ['Content-Type' => 'application/octet-stream'])
            ),
        ];
    }

    public function storeReference(string $uuid, int $registrationId, ?string $type = null): FileUpload
    {
        return FileUpload::create([
            'uuid' => $uuid,
            'type' => $type ?? null,
            'disk' => config('native-event-app.files.s3.bucket'),
            'path' => $this->getFilePath($uuid),
            'registration_id' => $registrationId,
        ]);
    }

    public function getStorageClient(): S3Client
    {
        return $this->storageClient();
    }

    protected function createCommand(S3Client $client, string $uuid): CommandInterface
    {
        return $client->getCommand('putObject', array_filter([
            'Bucket' => config('native-event-app.files.s3.bucket'),
            'Key' => $uuid,
            'ACL' => config('native-event-app.files.s3.visibility'),
            'ContentType' => 'application/octet-stream',
        ]));
    }

    private function getFilePath(string $uuid): string
    {
        throw_unless(
            $this->storageClient()->doesObjectExist(config('native-event-app.files.s3.bucket'), $uuid),
            FileNotFoundException::class
        );

        return $uuid;
    }

    private function storageClient(): S3Client
    {
        return new S3Client([
            'region' => config('native-event-app.files.s3.region'),
            'version' => 'latest',
            'signature_version' => 'v4',
            'credentials' => array_filter([
                'key' => config('native-event-app.files.s3.key'),
                'secret' => config('native-event-app.files.s3.secret'),
                'token' => config('native-event-app.files.s3.token'),
                'url' => config('native-event-app.files.s3.url'),
            ]),
        ]);
    }
}
