<?php

namespace Drp\NativeEventApp\Test\Unit\Console\Commands;

use Carbon\Carbon;
use Drp\NativeEventApp\Models\Event;
use Drp\NativeEventApp\Models\Notification;
use Drp\NativeEventApp\Notifications\Events\SendNotification;
use Drp\NativeEventApp\Test\TestCase;
use Illuminate\Console\Scheduling\Schedule;
use Illuminate\Support\Facades\Event as EventFacade;
use Illuminate\Support\Str;

class SendScheduledNotificationsTest extends TestCase
{
    /** @test */
    public function it_emits_the_send_notification_event_for_unsent_notifications(): void
    {
        Carbon::setTestNow('02-04-2020 14:00');
        $event = factory(Event::class)->create();
        EventFacade::fake();

        // Should send
        Notification::make($event, 'Title', 'A')
            ->sendAt(now())
            ->publish();

        factory(Notification::class)->state('published')->create([
            'title' => 'Title',
            'message' => 'B',
            'send_at' => now()->subHour(),
        ]);

        // Should not send
        $notificationC = Notification::make($event, 'Title', 'C')
            ->sendAt(now()->addHour())
            ->publish();

        $this->artisan('notifications:send');

        EventFacade::assertDispatched(SendNotification::class, 2);

        EventFacade::assertNotDispatched(
            SendNotification::class,
            static function (SendNotification $event) use ($notificationC) {
                return $event->notification->id === $notificationC->id;
            }
        );
    }

    /** @test */
    public function it_schedules_the_command_to_run_every_minute(): void
    {
        $schedule = app(Schedule::class);

        $commands = collect($schedule->events())->map(static function ($event): array {
            return [
                'command' => $event->command,
                'expression' => $event->expression,
            ];
        });

        $this->assertCount(1, $commands);
        $this->assertTrue(Str::contains($commands->first()['command'], 'notifications:send'));
        $this->assertEquals('* * * * *', $commands->first()['expression']);
    }
}
