<?php

namespace Drp\NativeEventApp\Test\Unit;

use Drp\NativeEventApp\Exceptions\InvalidPhoneNumber;
use Drp\NativeEventApp\PhoneNumber;
use Drp\NativeEventApp\Test\TestCase;

class PhoneNumberTest extends TestCase
{
    public function __setup()
    {
        //
    }

    public function phoneNumberProvider()
    {
        return [
            ['+44 (0) 7513 438167'],
            ['+44 (0) 7513438167'],
            ['44 (0) 7513 438167'],
            ['44 (0) 7513438167'],
            ['+44(0)7513438167'],
            ['+44 7513438167'],
            ['+4407513438167'],
            ['07513 438167'],
            ['07513 438167'],
            ['07513438167'],
            ['+44 7513 438167'],
            ['+447513438167'],
        ];
    }

    /**
     * @test
     * @dataProvider phoneNumberProvider
     */
    public function accepts_valid_phone_numbers(string $phoneNumber)
    {
        $this->assertInstanceOf(PhoneNumber::class, new PhoneNumber($phoneNumber));
    }

    /**
     * @test
     */
    public function does_not_accept_invalid_phone_numbers()
    {
        try {
            new PhoneNumber('12121');
        } catch (InvalidPhoneNumber $exception) {
            $this->assertStringContainsString('invalid', $exception->getMessage());
            return;
        }

        $this->fail('Exception was not thrown');
    }

    /**
     * @test
     */
    public function obfuscates_phone_number()
    {
        $phoneNumber = new PhoneNumber('07725811123');

        $this->assertEquals('123', $phoneNumber->obfuscated());
    }

    /**
     * @test
     * @dataProvider phoneNumberProvider
     */
    public function returns_international_number(string $phoneNumber)
    {
        $phoneNumber = new PhoneNumber($phoneNumber);
        $this->assertEquals('447513438167', $phoneNumber->internationalized());
    }

    /** @test */
    public function handles_zero_in_mobile_number()
    {
        $phoneNumber = new PhoneNumber('07707777777');
        $this->assertEquals('447707777777', $phoneNumber->internationalized());
    }
}
